#-*- coding:utf-8 -*-
import json
from Protocol import * 
from DataList import * 
from NMEA import *

class Antenna(Protocol):
    def __init__(self, name):
        Protocol.__init__(self, name)
        self.List = []
        self.Nmea = NmeaFormatter()
        self.Status = DataList()
        self.UpdateList();
        
    def DatProc(self,line):
        self.List.append(line)
        if len(self.List) > 4: self.List.pop(0)
        self.CheckRcv()
        lines = line.split(',')
        check = False
        for x in lines :
            self.Status.SetWithTag(x)
            
    def GetArray(self):
        dat = { }
        for i in self.Status.List:
            tmp = dict({i.Name : {"Value":i.Value,"Status":i.GetStatus()}})
            dat.update(tmp)
        return dat
    def GetJson(self):
        dat = { 
        # 'SyncOffset':0,
        'Antenna':{},
        'Nmea':{
            'Rmc':{},
            'Gga':{},
            }, 
        }
        
        # dat['SyncOffset'] = self.GetSyncStatus()
        dat['Antenna'] = self.GetArray()
        dat['Nmea']['Rmc'] = self.Nmea.Rmc.GetArray()
        dat['Nmea']['Gga'] = self.Nmea.Gga.GetArray()
        # return json.dumps(dat, sort_keys=True, indent=4)
        # return json.dumps(dat)
        return dat;
        
    def GetSyncStatus(self): 
        try: return 1 if self.SyncOffset > 0 else 0
        except : return 0
    def GetSyncOffset0(self): 
        try: return float(self.Status.GetWithToken('SyncOffset0'))
        except : return 0
    def GetSyncOffset1(self): 
        try: return float(self.Status.GetWithToken('SyncOffset1'))
        except : return 0
    def GetSyncOffset2(self): 
        try: return float(self.Status.GetWithToken('SyncOffset2'))
        except : return 0
    def GetGpsValid(self): 
        try: return 1 if int(self.Status.GetWithToken('GpsStatus')) == 2 else 0
        except : return 0
    def GetLatitude(self): 
        try: return float(self.Status.GetWithToken('Latitude'))
        except : return 0
    def GetLongitude(self): 
        try: return float(self.Status.GetWithToken('Longitude'))
        except : return 0
    def Pop(self):
        try :
            if len(self.List) > 1: return self.List.pop()
        except: self.out('* Antenna Pop Error ')
        return None
         
    def UpdateList(self):
        self.Status.Push(Data("Driving", DataType.Integer, "Aa","0"))
        self.Status.Push(Data("Static", DataType.Integer, "Ab","0"))
        self.Status.Push(Data("Searching", DataType.Integer, "An","0"))
        self.Status.Push(Data("Tracking", DataType.Integer, "Ao","0"))
        self.Status.Push(Data("TxMute", DataType.Integer, "As","0"))
        self.Status.Push(Data("AsiPass", DataType.Integer, "At","0"))
        self.Status.Push(Data("TrackingStatus", DataType.Integer, "Ad","0"))
        self.Status.Push(Data("DrivingStatus", DataType.Integer, "Ah","0"))
        self.Status.Push(Data("DrivingMode", DataType.Integer, "Ai","0"))
        self.Status.Push(Data("SearchingType", DataType.Integer, "Ae","0"))
        self.Status.Push(Data("TrackingType", DataType.Integer, "Af","0"))
        self.Status.Push(Data("ExternalSearchingType", DataType.Integer, "Xh","0"))
        self.Status.Push(Data("ExternalTrackingType", DataType.Integer, "Xi","0"))
        self.Status.Push(Data("ExternalLockThreshold", DataType.Integer, "Xj","0"))
        self.Status.Push(Data("GyroMode", DataType.Integer, "Ar","0"))
        self.Status.Push(Data("SkewMode", DataType.Integer, "Aj","0"))
        self.Status.Push(Data("JogMode", DataType.Integer, "Aq","0"))
        self.Status.Push(Data("TxBand", DataType.Integer, "Au","0"))
        self.Status.Push(Data("RxBand", DataType.Integer, "Ak","0"))
        self.Status.Push(Data("TxPolarity", DataType.Integer, "Av","0"))
        self.Status.Push(Data("RxPolarity", DataType.Integer, "Al","0"))
        self.Status.Push(Data("TRxPolarity", DataType.Integer, "Am","0"))
        # Satelite Information
        self.Status.Push(Data("SatNum", DataType.Integer, "Ba","0" ))
        self.Status.Push(Data("SatLon", DataType.Float, "Bb","0" ))
        self.Status.Push(Data("SatName", DataType.String, "Bl","0" ))
        self.Status.Push(Data("NID", DataType.Integer, "Bm","0" ))
        self.Status.Push(Data("Orbital", DataType.Integer, "Bn","0" ))
        # Tracking Status
        self.Status.Push(Data("NimLock", DataType.Integer, "Ac","0" ))
        self.Status.Push(Data("ModemLock", DataType.Integer, "Xd","0" ))
        self.Status.Push(Data("FunctionCode", DataType.Integer, "Yb","0" ))
        self.Status.Push(Data("ErrorCode", DataType.Integer, "Za","0" ))
        self.Status.Push(Data("PointingError", DataType.Float, "Xk","0" ))
        # Device Status
        self.Status.Push(Data("Temperature", DataType.Float, "Xa","0" ))
        # Device Status
        self.Status.Push(Data("NimAgc", DataType.Integer, "Bc","0" ))
        self.Status.Push(Data("NimQuality", DataType.Integer, "Bd","0" ))
        self.Status.Push(Data("Rssd", DataType.Integer, "Be","0" ))
        self.Status.Push(Data("RssdCn", DataType.Integer, "Bf","0" ))
        # LookAngle
        # 1. One hand
        self.Status.Push(Data("Azimuth", DataType.Float, "Cc","0" ))
        self.Status.Push(Data("Elevation", DataType.Float, "Cb","0" ))
        self.Status.Push(Data("Relative", DataType.Float, "Ed","0" ))
        self.Status.Push(Data("Skew", DataType.Float, "Ea","0" ))
        self.Status.Push(Data("Heading", DataType.Float, "Xc","0" ))
        # 2. Two hands
        # Yaw = Relative, RefAzimuth = Azimuth, RefElevation = Elevation;
        self.Status.Push(Data("Tilt", DataType.Float, "Eb","0" ))
        self.Status.Push(Data("Level", DataType.Float, "Ec","0" ))
        self.Status.Push(Data("RefBank", DataType.Float, "Ca","0" ))
        self.Status.Push(Data("AhrsRoll", DataType.Float, "Da","0" ))
        self.Status.Push(Data("AhrsPitch", DataType.Float, "Db","0" ))
        self.Status.Push(Data("AhrsYaw", DataType.Float, "Dc","0" ))
        # GPS
        self.Status.Push(Data("GpsStatus", DataType.Integer, "Ag","0" ))
        self.Status.Push(Data("Latitude", DataType.Float, "Bg","0" ))
        self.Status.Push(Data("Longitude", DataType.Float, "Bh","0" ))
        self.Status.Push(Data("GpsSpeed", DataType.Float, "Bj","0" ))
        self.Status.Push(Data("GpsTrackAngle", DataType.Float, "Bk","0" ))
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        