#-*- coding:utf-8 -*-
import pynmea2
import os
from Protocol import * 
from DataList import * 

class RMC(Protocol):
    def __init__(self, name):
        Protocol.__init__(self, name)
        self.Status = DataList()
        self.Status.Push(Data("Origin", DataType.String, "Uk","0"))
        self.Status.Push(Data("Time", DataType.Float, "Uk","0"))
        self.Status.Push(Data("Valid", DataType.String, "Uk","0"))
        self.Status.Push(Data("Latitude", DataType.Float, "Uk","0"))
        self.Status.Push(Data("LatDirection", DataType.String, "Uk","0"))
        self.Status.Push(Data("Longitude", DataType.Float, "Uk","0"))
        self.Status.Push(Data("LonDirection", DataType.String, "Uk","0"))
        self.Status.Push(Data("Speed", DataType.Float, "Uk","0"))
        self.Status.Push(Data("Date", DataType.Integer, "Uk","0"))
        self.Gll = GLL()
    def DatProc(self,line):
        self.Status.Set('Origin',line)
        if check_checksum(line)[0] != True : 
            Protocol.CheckCsErr(self)
            self.Status.Set('Valid','0')
            return 
        Protocol.CheckRcv(self)
        lines = line.split(',')
        # print line
        # print lines[2]
        self.Status.Set('Valid',lines[2])
        self.Status.Set('Time',lines[1])
        self.Status.Set('Valid',lines[2])
        self.Status.Set('Latitude',lines[3])
        self.Status.Set('LatDirection',lines[4])
        self.Status.Set('Longitude',lines[5])
        self.Status.Set('LonDirection',lines[6])
        self.Status.Set('Speed',lines[7])
        self.Status.Set('Date',lines[9])
        self.ResetRcvValid()
        self.Gll.SetRmc(lines)
        if self.IsValid() == True : self.UpdateGpstimeToSystime(lines);
        # self.Status.PrintAll()
        # print self.GetInterfaceStatusArray()
    def ToString(self):
        return self.Status.GetWithTokenObj('Origin').Value
    def GetLatitude(self):
        tmpstr = str(self.Status.GetWithTokenObj('LatDirection').Value)
        tmpsign = 1
        if len(tmpstr) > 0 :
            if tmpstr=="S" : tmpsign = -1
        tmpval = 0.0    
        tmpstr = str(self.Status.GetWithTokenObj('Latitude').Value)
        length = len(tmpstr)
        if length > 3 : 
            tmp_header = tmpstr[0:2]
            tmp_tail = tmpstr[2:length-1]
            tmpval = int(tmp_header) + (float(tmp_tail)/60)
        return tmpsign * tmpval
    def GetLongitude(self):
        tmpstr = str(self.Status.GetWithTokenObj('LonDirection').Value)
        tmpsign = 1
        if len(tmpstr) > 0 :
            if tmpstr=="W" : tmpsign = -1
        tmpval = 0.0    
        tmpstr = str(self.Status.GetWithTokenObj('Longitude').Value)
        length = len(tmpstr)
        if length > 3 : 
            tmp_header = tmpstr[0:3]
            tmp_tail = tmpstr[3:length-1]
            tmpval = int(tmp_header) + (float(tmp_tail)/60)
        return tmpsign * tmpval
    def IsValid(self):
        tmpstr = str(self.Status.GetWithTokenObj('Valid').Value)
        tmp = 0
        if len(tmpstr) > 0 : 
            if tmpstr=="A" or tmpstr == "1" or tmpstr == "2" : tmp = 1
        return tmp
    def GetUTC(self):
        return float(self.Status.GetWithTokenObj('Time').Value)
    def GetDatatime(self):
        tmp_time = str(self.Status.GetWithTokenObj('Time').Value)
        tmp_date = str(self.Status.GetWithTokenObj('Date').Value)
        if tmp_time == '0' or tmp_date == '0' : return '2000-00-00 00:00:00.00'
        YY = tmp_date[4:6]
        MM = tmp_date[2:4]
        DD = tmp_date[0:2]
        hh = tmp_time[0:2]
        mm = tmp_time[2:4]
        if len(tmp_time) > 9 : ss = tmp_time[4:9]
        else : ss = tmp_time[4:6]
        return '20'+YY+'-'+MM+'-'+DD+'-'+hh+':'+mm+':'+ss 
    def GetSpeed(self):
        return float(self.Status.GetWithTokenObj('Speed').Value)
    def GetArray(self):
        dat = { 
            Protocol.GetName(self):{
                'Origin': self.ToString(),
                'Datetime': self.GetDatatime(),
                'Valid': self.IsValid(),
                'Latitude': self.GetLatitude(),
                'Longitude': self.GetLongitude(),
                'Speed': self.GetSpeed(),
                'Count': self.Count(),
                'CsErrCount': self.CsErr(),
            },
        }
        return dat
    def GetOutput(self, opt):	
        if len(self.Status.GetWithTokenObj('Origin').Value) < 5 : return ''
        dat = self.Status.GetWithTokenObj('Origin').Value
        lines = dat.split(',')
        tmp = pynmea2.RMC(opt, 'RMC', (lines[1],lines[2],lines[3],lines[4],lines[5],lines[6],lines[7],lines[8],lines[9],lines[10],lines[11]))
        return tmp
    def GetGprmc(self):	return self.GetOutput('GP')
    def GetGnrmc(self):	return self.GetOutput('GN')
    def UpdateGpstimeToSystime(self,lines):
        from Project import pj 
        reval = os.popen('date +%H:%M:%S').read()
        # print reval
        syst = reval.split(':')
        syst_ = (int(syst[0])*3600+int(syst[1])*60+int(syst[2]))
        gpst_ = (int(lines[1][0:2])*3600+int(lines[1][2:4])*60+int(lines[1][4:6]))
        # print str(syst_) + " - " + str(gpst_)
        if abs(syst_ - gpst_) > 30 and pj.UpdateSysTimeCnt < 2: 
            tmp = lines[9][2:4];
            tmp += lines[9][0:2];
            tmp += lines[1][0:4];
            tmp += "20";
            tmp += lines[9][4:6];
            tmp += ".";
            tmp += lines[1][4:6];
            cmd = "sudo date "+ tmp
            reval = os.popen(cmd).read()
            if reval[len(reval)-1] == '\n' : reval = reval[:-1]
            if reval[len(reval)-1] == '\r' : reval = reval[:-1]
            print("* Updated SystemTime : {}".format(reval))
            pj.UpdateSysTimeCnt = 600
    def GetGga(self, opt):	
        if len(self.Status.GetWithTokenObj('Origin').Value) < 5 : return ''
        dat = self.Status.GetWithTokenObj('Origin').Value
        lines = dat.split(',')
        return pynmea2.GGA(opt, 'GGA', (lines[1],lines[3],lines[4],lines[5],lines[6],lines[2],"","","","","","","",""))
    def GetGpgga(self):	return self.GetOutput('GP')
    def GetGngga(self):	return self.GetOutput('GN')
    
class GGA(Protocol):
    def __init__(self, name):
        Protocol.__init__(self, name)
        self.Status = DataList()
        self.Status.Push(Data("Origin", DataType.String, "Uk","0"))
        self.Status.Push(Data("Latitude", DataType.Float, "Uk","0"))
        self.Status.Push(Data("LatDirection", DataType.String, "Uk","0"))
        self.Status.Push(Data("Longitude", DataType.Float, "Uk","0"))
        self.Status.Push(Data("LonDirection", DataType.String, "Uk","0"))
        self.Status.Push(Data("Valid", DataType.Integer, "Uk","0"))
        self.Status.Push(Data("SatelliteNumber", DataType.Integer, "Uk","0"))
        self.Status.Push(Data("Altitude", DataType.Float, "Uk","0"))
        self.Gll = GLL()
    def DatProc(self,line):
        self.Status.Set('Origin',line)
        if check_checksum(line)[0] != True : 
            Protocol.CheckCsErr(self)
            self.Status.Set('Valid','0')
            return
        Protocol.CheckRcv(self)
        lines = line.split(',')
        self.Status.Set('Time',lines[1])
        self.Status.Set('Latitude',lines[2])
        self.Status.Set('LatDirection',lines[3])
        self.Status.Set('Longitude',lines[4])
        self.Status.Set('LonDirection',lines[5])
        self.Status.Set('Valid',lines[6])
        self.Status.Set('SatelliteNumber',lines[7])
        self.Status.Set('Altitude',lines[9])
        self.ResetRcvValid()
        self.Gll.SetGga(lines)
        # self.Status.PrintAll()
        # print self.GetInterfaceStatusArray()
    def ToString(self):
        return self.Status.GetWithTokenObj('Origin').Value
    def GetLatitude(self):
        tmpstr = str(self.Status.GetWithTokenObj('LatDirection').Value)
        tmpsign = 1
        if len(tmpstr) > 0 :
            if tmpstr=="S" : tmpsign = -1
        tmpval = 0.0    
        tmpstr = str(self.Status.GetWithTokenObj('Latitude').Value)
        length = len(tmpstr)
        if length > 3 : 
            tmp_header = tmpstr[0:2]
            tmp_tail = tmpstr[2:length-1]
            tmpval = int(tmp_header) + (float(tmp_tail)/60)
        return tmpsign * tmpval
    def GetLongitude(self):
        tmpstr = str(self.Status.GetWithTokenObj('LonDirection').Value)
        tmpsign = 1
        if len(tmpstr) > 0 :
            if tmpstr=="W" : tmpsign = -1
        tmpval = 0.0    
        tmpstr = str(self.Status.GetWithTokenObj('Longitude').Value)
        length = len(tmpstr)
        if length > 3 : 
            tmp_header = tmpstr[0:3]
            tmp_tail = tmpstr[3:length-1]
            tmpval = int(tmp_header) + (float(tmp_tail)/60)
        return tmpsign * tmpval
    def IsValid(self):
        tmpstr = str(self.Status.GetWithTokenObj('Valid').Value)
        tmp = 0
        if len(tmpstr) > 0 : 
            if tmpstr=="A" or tmpstr == "1" or tmpstr == "2" : tmp = 1
        return tmp
    def GetAltitude(self):
        return float(self.Status.GetWithTokenObj('Altitude').Value)
    def GetUTC(self):
        return float(self.Status.GetWithTokenObj('Time').Value)
    def GetSatNum(self):
        return int(self.Status.GetWithTokenObj('SatelliteNumber').Value)
    def GetArray(self):
        dat = { 
            Protocol.GetName(self):{
                'Origin': self.ToString(),
                'Valid': self.IsValid(),
                'Latitude': self.GetLatitude(),
                'Longitude': self.GetLongitude(),
                'Altitude': self.GetAltitude(),
                'SatelliteNumber': self.GetSatNum(),
                'Count': self.Count(),
                'CsErrCnt': self.CsErr(),
            },
        }
        return dat
    def GetOutput(self, opt):	
        if len(self.Status.GetWithTokenObj('Origin').Value) < 5 : return ''
        dat = self.Status.GetWithTokenObj('Origin').Value
        lines = dat.split(',')
        return pynmea2.GGA(opt, 'GGA', (lines[1],lines[2],lines[3],lines[4],lines[5],lines[6],lines[7],lines[8],lines[9],lines[10],lines[11],lines[12],lines[13],lines[14]))
    def GetGpgga(self):	return self.GetOutput('GP')
    def GetGngga(self):	return self.GetOutput('GN')
    def GetRmc(self, opt):	
        if len(self.Status.GetWithTokenObj('Origin').Value) < 5 : return ''
        dat = self.Status.GetWithTokenObj('Origin').Value
        lines = dat.split(',')
        tmp = pynmea2.RMC(opt, 'RMC', (lines[1],lines[6],lines[2],lines[3],lines[4],lines[5],"","","","",""))
        return tmp
    def GetGprmc(self):	return self.GetOutput('GP')
    def GetGnrmc(self):	return self.GetOutput('GN')
		
class HD_T(Protocol):
    def __init__(self, name):
        Protocol.__init__(self, name)
        self.Status = DataList()
        self.Status.Push(Data("Origin", DataType.String, "Uk","0"))
        self.Status.Push(Data("Heading", DataType.Float, "Uk","0"))
    def DatProc(self,line):
        self.Status.Set('Origin',line)
        if check_checksum(line)[0] != True : 
            Protocol.CheckCsErr(self)
            self.Status.Set('Valid','0')
            return
        Protocol.CheckRcv(self)
        lines = line.split(',')
        self.Status.Set('Origin',line)
        self.Status.Set('Heading',lines[1])
        self.ResetRcvValid()
        # print self.GetInterfaceStatusArray()
    def ToString(self):
        return self.Status.GetWithTokenObj('Origin').Value
    def GetHeading(self):
        return float(self.Status.GetWithTokenObj('Heading').Value)
    def GetArray(self):
        dat = { 
            Protocol.GetName(self):{
                'Origin': self.ToString(),
                'Heading': self.GetHeading(),
                'Count': self.Count(),
                'CsErrCnt': self.CsErr(),
            },
        }
        return dat
        
class GLL():
    def __init__(self):
        self.UTC = '';
        self.Valid = 'V';
        self.Latitude = '';
        self.LatDirection = '';
        self.Longitude = '';
        self.LonDirection = '';
    def SetRmc(self, lines):	
        self.UTC = lines[1];
        self.Valid = lines[2];
        self.Latitude = lines[3];
        self.LatDirection = lines[4];
        self.Longitude = lines[5];
        self.LonDirection = lines[6];
    def SetGga(self, lines):	
        self.UTC = lines[1];
        self.Latitude = lines[2];
        self.LatDirection = lines[3];
        self.Longitude = lines[4];
        self.LonDirection = lines[5];
        self.Valid = 'A' if lines[6] == '1' else 'V'
    def GetOutput(self, opt):	
        return pynmea2.GLL(opt, 'GLL', (self.Latitude, self.LatDirection, self.Longitude, self.LonDirection, self.UTC, self.Valid))
    def GetGpgll(self):	return self.GetOutput('GP')
    def GetGngll(self):	return self.GetOutput('GN')
    
class NmeaFormatter: 
    def __init__(self):
        self.Rmc = RMC('RMC') 
        self.Gga = GGA('GGA') 
        self.Hdt = HD_T('HDT') 
        self.Htg = HD_T('HTG') 
        self.Htm = HD_T('HTM') 
        # self.Gll = GLL()
        # self.GllFromAnt = GLL()
        self.Heading = 0.0
        p = Thread(target=self.Worker)
        p.start()
    def DatProc(self,line):
        lines = line.split(',')
        if lines[0][3:6] == 'RMC' :  self.Rmc.DatProc(line) #, self.Gll.SetRmc(lines)
        elif lines[0][3:6] == 'GGA' : self.Gga.DatProc(line) #, self.Gll.SetGga(lines)
        elif lines[0][3:6] == 'HDT' : 
            self.Hdt.DatProc(line)
            self.Heading = self.Hdt.GetHeading()
        elif lines[0][3:6] == 'HDG' : 
            self.Htg.DatProc(line)
            self.Heading = self.Htg.GetHeading()
        elif lines[0][3:6] == 'HDM' : 
            self.Htm.DatProc(line)
            self.Heading = self.Htm.GetHeading()
    def RcvCount(self):
        return (self.Rmc.RcvCnt+self.Gga.RcvCnt+self.Hdt.RcvCnt+self.Htg.RcvCnt+self.Htm.RcvCnt)  
    def BurstGpsProc(self,line):
        lines = line.split('$')
        rmc = lines[2].split(',')
        gga = lines[3].split(',')
        if rmc[0][2:5] == 'RMC' : self.Rmc.DatProc('$'+lines[2][:len(lines[2])-1])
        if gga[0][2:5] == 'GGA' : self.Gga.DatProc('$'+lines[3][:len(lines[3])-1])
        
        # if rmc[0][2:5] == 'RMC' : self.Rmc.DatProc('$GPRMC,092750.000,A,5321.6802,N,00630.3372,W,0.02,31.66,280511,,,A*43')
        # if gga[0][2:5] == 'GGA' : self.Gga.DatProc('$GPGGA,092751.000,5321.6802,N,00630.3371,W,1,8,1.03,61.7,M,55.3,M,,*75')
    def Worker(self):
        # from Project import pj 
        while True:
            time.sleep(1)
            self.Rmc.CheckRcvValid()    
            self.Gga.CheckRcvValid()    
            self.Hdt.CheckRcvValid()    
            self.Htg.CheckRcvValid()  
            self.Htm.CheckRcvValid()
            #during time = 20min
            # if pj.UpdateSysTimeCnt > 0 : pj.UpdateSysTimeCnt -= 1
            
    def IsValidRmc(self): 
        return self.Rmc.RcvValid() and self.Rmc.IsValid()
    def IsValidGga(self): 
        return self.Gga.RcvValid() and self.Gga.IsValid()
    def IsValidHdt(self): 
        return self.Hdt.RcvValid()
    def IsValidHtg(self): 
        return self.Htg.RcvValid()
    def IsValidHtm(self): 
        return self.Htm.RcvValid()
    def GetValidArray(self):
        dat = { 
            'RcvCount':{},
            'RcvValid':{
                'Rmc':self.Rmc.RcvValid(),
                'Gga':self.Gga.RcvValid(),
                'Hdt':self.Hdt.RcvValid(),
                'Htg':self.Htg.RcvValid(),
                'Htm':self.Htm.RcvValid(),
            },
        }
        dat['RcvCount'] = self.RcvCount()
        return dat
    def GetArray(self):
        dat = { 
            'RcvCount':{},
            'Rmc':{},
            'Gga':{},
            'Hdt':{},
            'Htg':{},
            'Htm':{},
            'RcvValid':{
                'Rmc':self.Rmc.RcvValid(),
                'Gga':self.Gga.RcvValid(),
                'Hdt':self.Hdt.RcvValid(),
                'Htg':self.Htg.RcvValid(),
                'Htm':self.Htm.RcvValid(),
            },
        }
        dat['RcvCount'] = self.RcvCount()
        dat['Rmc'] = self.Rmc.GetArray()
        dat['Gga'] = self.Gga.GetArray()
        dat['Hdt'] = self.Hdt.GetArray()
        dat['Htg'] = self.Htg.GetArray()
        dat['Htm'] = self.Htm.GetArray()
        return dat
        
"""
   NMEA string parser class
   by Brian C. Lane <bcl@brianlane.com>
   Copyright 2006 by Brian C. Lane
   All Rights Reserved
"""

def check_checksum( msg ):
    """
    Check the checksum of a NMEA 0183 sentence. Skips leading ! or $ and stops
    at *
    It ignores non-standard extensions after the checksum
    It ignores anything before the first $ or ! in the string
    """

    # Find the start of the NMEA sentence
    startchars = "!$"
    for c in startchars:
        i = msg.find(c)
        if i >= 0: break
    else:
        return (False, None, None)

    # Calculate the checksum on the message
    sum1 = 0
    for c in msg[i+1:]:
        if c == '*':
            break
        sum1 = sum1 ^ ord(c)
    sum1 = sum1 & 0xFF

    # Find the checksum at the end of the string
    sum2 = -1
    try:
        i = msg.index( '*' )
        sum2 = int(msg[i+1:i+3], 16)

        if sum1 == sum2:
            return (True, sum1, sum2)
    except:
        pass

    return (False, sum1, sum2)


def make_checksum( msg ):
    """
    Make a NMEA 0183 checksum on a string. Skips leading ! or $ and stops
    at *
    It ignores anything before the first $ or ! in the string
    """

    # Find the start of the NMEA sentence
    startchars = "!$"
    for c in startchars:
        i = msg.find(c)
        if i >= 0: break
    else:
        return (False, None, None)

    # Calculate the checksum on the message
    sum1 = 0
    for c in msg[i+1:]:
        if c == '*':
            break
        sum1 = sum1 ^ ord(c)
    sum1 = sum1 & 0xFF

    return sum1    

def GetHdtFormat(_heading) :
    tmp_headig = "$HEHDT,%05.1f,T"%_heading
    tmp = "%s*%X"%(tmp_headig, make_checksum(tmp_headig))
    return tmp
        
        
        
    